unit rbEditTemplatesAsText;

interface

uses


{$DEFINE DADE}            {remove the 'x' to enable DADE}
{$DEFINE BDE}             {remove the 'x' to enable Borland Database Engine (BDE) }
{x$DEFINE ADO}            {remove the 'x' to enable ADO}
{x$DEFINE IBExpress}      {remove the 'x' to enable Interbase Express}
{$DEFINE CrossTab}        {remove the 'x' to enable CrossTab}
{x$DEFINE RAP}            {remove the 'x' to enable RAP}
{x$DEFINE CheckBox}       {remove the 'x' to enable CheckBox}
{x$DEFINE TeeChart}       {remove the 'x' to enable standard TeeChart}
{x$DEFINE UseDesignPath}  {remove the 'x' to use the design-time settings of Database object on this form}


{$IFDEF DADE}
  daIDE, 
{$ENDIF}

{$IFDEF BDE}
  daDBBDE,
{$ENDIF}

{$IFDEF ADO}
  daADO,
{$ENDIF}

{$IFDEF IBExpress}
  daIBExpress,
{$ENDIF}

{$IFDEF CrossTab}
  ppCTDsgn,
{$ENDIF}

{$IFDEF RAP}
  raIDE,
{$ENDIF}

{$IFDEF CheckBox}
  myChkBox,
{$ENDIF}

{$IFDEF TeeChart}
  ppChrt, ppChrtDP,
{$ENDIF}
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ppBands, ppCache, ppClass, ppProd, ppReport, Db, DBTables,
  ppComm, ppRelatv, ppDB, ppDBPipe, ppDBBDE, ppTypes;



type
  TForm1 = class(TForm)
    tblItem: TTable;
    dsItem: TDataSource;
    euDatabase: TDatabase;
    lbxItems: TListBox;
    btnSave: TButton;
    Memo1: TMemo;
    plItem: TppDBPipeline;
    Memo2: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure lbxItemsClick(Sender: TObject);
  private
    procedure PopulateListBox;
    procedure LoadTemplate(aItemIndex: Integer);
    procedure SaveTemplate;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

{------------------------------------------------------------------------------}
{ TForm1.FormCreate }

procedure TForm1.FormCreate(Sender: TObject);
begin
  PopulateListBox;
  lbxItems.ItemIndex := 0;
  LoadTemplate(0);
end;

{------------------------------------------------------------------------------}
{ TForm1.lbxItemsClick }

procedure TForm1.lbxItemsClick(Sender: TObject);
begin
  LoadTemplate(lbxItems.ItemIndex);

end;

{------------------------------------------------------------------------------}
{ TForm1.btnSaveClick }

procedure TForm1.btnSaveClick(Sender: TObject);
begin
  SaveTemplate;
end;

{------------------------------------------------------------------------------}
{ TForm1.PopulateListBox }

procedure TForm1.PopulateListBox;
const
  cReport = 1;
var
  liItemID: Integer;
begin

  plItem.Open;
  plItem.First;

  while not plItem.EOF do
    begin
      {check that item type is a report}
      if plItem['ItemType'] = Ord(cReport) then
        begin
          {load the name and the ItemID to the list box}
          liItemID := plItem['ItemID'];
          lbxItems.Items.AddObject(plItem['Name'], TObject(liItemID));
        end;

      plItem.Next;

    end;


end;



{------------------------------------------------------------------------------}
{ TForm1.LoadTemplate }

procedure TForm1.LoadTemplate(aItemIndex: Integer);
var
  liItemID: Integer;
  lBinaryStream: TMemoryStream;
  lTextStream: TMemoryStream;
begin

  {get the item id for the selected item}
  liItemID := Integer(lbxItems.Items.Objects[aItemIndex]);

  {locate the record to edit}
  if not plItem.Locate('ItemID', liItemID, []) then
    raise Exception.Create('Unable to locate selected report');


  lBinaryStream := TMemoryStream.Create;
  lTextStream := TMemoryStream.Create;

  try

    {read the binary stream from the database}
    plItem.GetFieldAsStream('Template', lBinaryStream);

    lBinaryStream.Position := 0;

    {convert the binary stream to text}
    ObjectBinaryToText(lBinaryStream, lTextStream);

    lTextStream.Position := 0;

    {load the text stream to the memo control}
    memo1.Lines.LoadFromStream(lTextStream);


  finally
    lBinaryStream.Free;
    lTextStream.Free;
  end;


end;

{------------------------------------------------------------------------------}
{ TForm1.SaveTemplate }

procedure TForm1.SaveTemplate;
var
  lBinaryStream: TMemoryStream;
  lTextStream: TMemoryStream;
  lWriter: TWriter;
begin

  lBinaryStream := TMemoryStream.Create;
  lTextStream := TMemoryStream.Create;

  try

    {load the text stream to the memo control}
    memo1.Lines.SaveToStream(lTextStream);

    lTextStream.Position := 0;

    {convert the binary stream to text}
    ObjectTextToBinary(lTextStream, lBinaryStream);

    lBinaryStream.Seek(0,soFromEnd);
    lWriter := TWriter.Create(lBinaryStream, 1024);

    try
      lWriter.Root := Self;
      lWriter.WriteListEnd;
    finally
      lWriter.Free;
    end;

    lBinaryStream.Position := 0;

    plItem.Edit;

    {load the binary stream from the database}
    plItem.SetFieldFromStream('Template', lBinaryStream);

    plItem.Post;

  finally
    lBinaryStream.Free;
    lTextStream.Free;
  end;

end;





end.
